@@ -14,15 +14,17 @@ import time
 import tempfile
 import uuid
 import base64
+from Orange.data import Table, StringVariable, Domain
+import platform
 
 
 
 if "site-packages/Orange/widgets" in os.path.dirname(os.path.abspath(__file__)).replace("\\", "/"):
     from Orange.widgets.orangecontrib.AAIT.utils import (SimpleDialogQt, aait_repo_file, shared_variables,
-                     windows_utils)
+                     windows_utils,mac_utils)
 else:
     from orangecontrib.AAIT.utils import (SimpleDialogQt, aait_repo_file, shared_variables,
-                     windows_utils)
+                     windows_utils,mac_utils)
 
 
 def force_save(self_instance):
@@ -456,8 +458,20 @@ def get_local_store_path():
     none_standard_path = sys.executable.replace("\\", "/")
     none_standard_path = os.path.dirname(none_standard_path)
     none_standard_path = os.path.dirname(none_standard_path)+"/aait_store/"
+
+
+
+
     if os.path.isfile(none_standard_path+"remote_ressources_path.txt"):
         store_ia_path=none_standard_path
+    elif  os.path.isdir(none_standard_path):
+        try:
+            with open(none_standard_path + '/remote_ressources_path.txt', 'w'):
+                pass
+            store_ia_path=none_standard_path
+        except:
+            SimpleDialogQt.BoxError("error impossible to write file :" + none_standard_path)
+            raise Exception("error impossible to write file :" + none_standard_path)
 
     if not os.path.exists(store_ia_path):
         # Create a new directory because it does not exist
@@ -856,6 +870,11 @@ def download_aait_store_file_or_directory(server_path: str = "", target: str = "
                 if os.path.isdir(server_path):
                     output=str(Path(output).resolve().parent).replace('\\','/')+"/"
                 windows_utils.win32_shellcopy(source, output)
+            elif platform.system() == "Darwin":
+                if os.path.isdir(server_path):
+                    output=str(Path(output).resolve().parent).replace('\\','/')+"/"
+                mac_utils.mac_copy_with_anyqt_progress(source, output)
+
             else:
                 #try:
                 #    windows_utils.mac_shellcopy(source, output)
@@ -1357,10 +1376,24 @@ def get_second_from_1970():
 
 def write_file_time(path):
     time= get_second_from_1970()
-    time_ok=path[:-3]+".ok"
+    time_ok=path[:-4]+".ok"
     reset_files([time_ok])
     with open(path, "w") as f:
         f.write(str(time))
+        f.flush()
+    try:
+        with open(time_ok, "w"):
+            pass
+    except Exception as e:
+        print(f"Error creating .ok file: {e}")
+    return
+def write_file_arbitrary_time(path,time):
+    time= int(time)
+    time_ok=path[:-4]+".ok"
+    reset_files([time_ok])
+    with open(path, "w") as f:
+        f.write(str(time))
+        f.flush()
     try:
         with open(time_ok, "w"):
             pass
@@ -1369,7 +1402,7 @@ def write_file_time(path):
     return
 
 def read_file_time(path):
-    time_ok = path[:-3] + ".ok"
+    time_ok = path[:-4] + ".ok"
     for _ in range(100):
         if not os.path.exists(time_ok):
                 time.sleep(0.5)
@@ -1377,6 +1410,8 @@ def read_file_time(path):
         return 0 # time out
     with open(path, "r") as f:
         content = f.read()
+    if content.isdigit()==False:
+        return int(get_second_from_1970()) # en cas d'erreur on renvoie le temps actuel
     return int(content)
 
 def generate_unique_id_from_mac_timestamp():
@@ -1392,7 +1427,50 @@ def generate_unique_id_from_mac_timestamp():
     out= str(base64.b32encode(raw_bytes).decode().rstrip('=').lower())
     return out
 
+def describe_orange_table(data_table):
+    """
+    Décrit une Orange.data.Table.
+    Retour:
+      - None en cas d'erreur (mauvais type, exception, etc.)
+      - (n_rows, columns) sinon, où:
+          n_rows: int, nombre de lignes
+          columns: liste de dicts {name, kind, var_type}
+                   - kind ∈ {"feature","target","meta"}
+                   - var_type ∈ {"StringVariable","ContinuousVariable","Categorical","TimeVariable", ...}
+    """
+    try:
+        if not isinstance(data_table, Table):
+            return None
 
+        def var_type_name(var):
+            return type(var).__name__         # fallback propre
+
+        n_rows = len(data_table)
+        dom = data_table.domain
+
+        cols = []
+        # Features
+        for v in dom.attributes:
+            cols.append({"name": v.name, "kind": "feature", "var_type": var_type_name(v)})
+        # Targets (peut être 0, 1 ou plusieurs class_vars)
+        for v in dom.class_vars:
+            cols.append({"name": v.name, "kind": "target", "var_type": var_type_name(v)})
+        # Metas
+        for v in dom.metas:
+            cols.append({"name": v.name, "kind": "meta", "var_type": var_type_name(v)})
+
+        return n_rows, cols
+
+    except Exception as e:
+        print(e)
+        return None
+
+
+def create_trigger_table():
+    var = StringVariable("Trigger")
+    dom = Domain([], metas=[var])
+    table = Table.from_list(domain=dom, rows=[["Trigger"]])
+    return table
 
 
 if __name__ == "__main__":
