@@ -13,6 +13,10 @@ if os.name=='nt':
     from win32com.shell import shell, shellcon
     import ctypes
     import ctypes.wintypes
+    from ctypes import windll, c_wchar_p
+    import ctypes.wintypes as wt
+
+
     def win32_shellcopy(src, dest):
         """
         Copy files and directories using Windows shell.
@@ -54,139 +58,92 @@ if os.name=='nt':
 
         return not aborted
 
-    def select_folder_ctypes():
+
+    def select_folder_ctypes(title="Select a directory"):
         """
-        Opens a native Windows folder selection dialog with enforced modal-like behavior, without attaching to any parent window.
-
-        Features:
-        - Uses the Windows Shell API (SHBrowseForFolder) to display the folder picker.
-        - Detects and brings the folder selection window to the foreground immediately after opening.
-        - Continuously forces the dialog window to stay topmost until a folder is selected or the dialog is closed.
-        - Ensures Unicode support for folder paths (handles accents and special characters).
-        - Properly frees allocated memory (PIDL) after selection.
-
-        Note:
-        - The dialog is not attached to any application window (no HWND owner is specified).
-
-        Returns:
-            str: The selected folder path, or an empty string if no selection was made.
+        Ouvre une boîte de sélection de dossier Windows toujours au premier plan
+        (devant toutes les fenêtres du système), sans threads ni repop.
         """
+
         BIF_RETURNONLYFSDIRS = 0x0001
         BIF_NEWDIALOGSTYLE = 0x0040
-        MAX_PATH = 260
+        MAX_PATH = 32767
 
+        BFFM_INITIALIZED = 1
         SWP_NOMOVE = 0x0002
         SWP_NOSIZE = 0x0001
         SWP_SHOWWINDOW = 0x0040
         HWND_TOPMOST = -1
 
+        SW_SHOW = 5
+
+        user32 = ctypes.windll.user32
+        shell32 = ctypes.windll.shell32
+        ole32 = ctypes.windll.ole32
+
         class BROWSEINFO(ctypes.Structure):
             _fields_ = [
-                ("hwndOwner", ctypes.wintypes.HWND),
+                ("hwndOwner", wt.HWND),
                 ("pidlRoot", ctypes.c_void_p),
                 ("pszDisplayName", ctypes.c_wchar_p),
                 ("lpszTitle", ctypes.c_wchar_p),
-                ("ulFlags", ctypes.wintypes.UINT),
+                ("ulFlags", wt.UINT),
                 ("lpfn", ctypes.c_void_p),
                 ("lParam", ctypes.c_void_p),
-                ("iImage", ctypes.wintypes.INT),
+                ("iImage", wt.INT),
             ]
 
-        user32 = ctypes.windll.user32
-        #kernel32 = ctypes.windll.kernel32
-        shell32 = ctypes.windll.shell32
+        BFFCALLBACK = ctypes.WINFUNCTYPE(wt.INT, wt.HWND, wt.UINT, wt.LPARAM, wt.LPARAM)
 
         SHBrowseForFolderW = shell32.SHBrowseForFolderW
         SHGetPathFromIDListW = shell32.SHGetPathFromIDListW
-        CoTaskMemFree = ctypes.windll.ole32.CoTaskMemFree
-
-        SHGetPathFromIDListW.argtypes = [ctypes.c_void_p, ctypes.c_wchar_p]
-        SHGetPathFromIDListW.restype = ctypes.wintypes.BOOL
+        CoTaskMemFree = ole32.CoTaskMemFree
+        CoInitializeEx = ole32.CoInitializeEx
+        CoUninitialize = ole32.CoUninitialize
 
         SHBrowseForFolderW.argtypes = [ctypes.POINTER(BROWSEINFO)]
         SHBrowseForFolderW.restype = ctypes.c_void_p
+        SHGetPathFromIDListW.argtypes = [ctypes.c_void_p, ctypes.c_wchar_p]
+        SHGetPathFromIDListW.restype = wt.BOOL
 
-        # Avant : liste des fenêtres existantes
-        hwnd_list_before = []
-
-        def enum_windows_proc(hwnd, lParam):
-            hwnd_list_before.append(hwnd)
-            return True
-
-        EnumWindows = user32.EnumWindows
-        EnumWindowsProc = ctypes.WINFUNCTYPE(ctypes.wintypes.BOOL, ctypes.wintypes.HWND, ctypes.wintypes.LPARAM)
-        EnumWindows(EnumWindowsProc(enum_windows_proc), 0)
-
-        # Préparer browseinfo
-        display_name_buffer = ctypes.create_unicode_buffer(MAX_PATH)
-
-        browse_info = BROWSEINFO()
-        browse_info.hwndOwner = 0
-        browse_info.pidlRoot = None
-        browse_info.pszDisplayName = ctypes.cast(display_name_buffer, ctypes.c_wchar_p)
-        browse_info.lpszTitle = "Select a directory"
-        browse_info.ulFlags = BIF_RETURNONLYFSDIRS | BIF_NEWDIALOGSTYLE
-        browse_info.lpfn = None
-        browse_info.lParam = None
-        browse_info.iImage = 0
-
-        # Ouvrir boîte de dialogue
-        pidl = SHBrowseForFolderW(ctypes.byref(browse_info))
-
-        # Petit délai pour affichage
-        time.sleep(0.1)
-
-        # Après ouverture : détecter la nouvelle fenêtre
-        hwnd_list_after = []
-
-        def enum_windows_proc_after(hwnd, lParam):
-            hwnd_list_after.append(hwnd)
-            return True
-
-        EnumWindows(EnumWindowsProc(enum_windows_proc_after), 0)
-
-        new_hwnds = list(set(hwnd_list_after) - set(hwnd_list_before))
-
-        # Identifier la fenêtre de sélection
-        selector_hwnd = None
-        for hwnd in new_hwnds:
-            length = user32.GetWindowTextLengthW(hwnd)
-            if length > 0:
-                title = ctypes.create_unicode_buffer(length + 1)
-                user32.GetWindowTextW(hwnd, title, length + 1)
-                if "Select a directory" in title.value or "Browse For Folder" in title.value:
-                    selector_hwnd = hwnd
-                    break
-
-        # 💥 Nouvelle partie : boucle qui surveille la fenêtre
-        def enforce_modal(hwnd):
-            if not hwnd:
-                return
-            while user32.IsWindow(hwnd):
+        def _cb(hwnd, uMsg, lParam, lpData):
+            if uMsg == BFFM_INITIALIZED:
+                # Force la fenêtre au tout premier plan
+                user32.ShowWindow(hwnd, SW_SHOW)
                 user32.SetForegroundWindow(hwnd)
-                user32.SetWindowPos(hwnd, HWND_TOPMOST, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE | SWP_SHOWWINDOW)
-                time.sleep(0.2)  # Toutes les 200 ms on remet focus
+                user32.SetActiveWindow(hwnd)
+                user32.BringWindowToTop(hwnd)
+                user32.SetWindowPos(hwnd, HWND_TOPMOST, 0, 0, 0, 0,
+                                    SWP_NOMOVE | SWP_NOSIZE | SWP_SHOWWINDOW)
+            return 0
 
-        if selector_hwnd:
-            threading.Thread(target=enforce_modal, args=(selector_hwnd,), daemon=True).start()
+        cb = BFFCALLBACK(_cb)
+
+        COINIT_APARTMENTTHREADED = 0x2
+        CoInitializeEx(None, COINIT_APARTMENTTHREADED)
+
+        display_name_buffer = ctypes.create_unicode_buffer(MAX_PATH)
+        bi = BROWSEINFO()
+        bi.hwndOwner = 0  # pas d’attachement à une autre fenêtre
+        bi.pidlRoot = None
+        bi.pszDisplayName = ctypes.cast(display_name_buffer, ctypes.c_wchar_p)
+        bi.lpszTitle = title
+        bi.ulFlags = BIF_RETURNONLYFSDIRS | BIF_NEWDIALOGSTYLE
+        bi.lpfn = ctypes.cast(cb, ctypes.c_void_p)
+        bi.lParam = None
+        bi.iImage = 0
+
+        pidl = SHBrowseForFolderW(ctypes.byref(bi))
 
-        # Lecture du résultat
         folder_path = ""
-
         if pidl:
-            path_buffer = ctypes.create_unicode_buffer(MAX_PATH)
-            success = SHGetPathFromIDListW(pidl, path_buffer)
-            if success:
-                folder_path = path_buffer.value
-            else:
-                folder_path=""
-
+            out_buf = ctypes.create_unicode_buffer(MAX_PATH)
+            if SHGetPathFromIDListW(pidl, out_buf):
+                folder_path = out_buf.value
             CoTaskMemFree(ctypes.c_void_p(pidl))
-        folder_path=folder_path.replace("\\","/")
-        if folder_path:
-            return folder_path
-        else:
-            return ""
+
+        CoUninitialize()
+        return folder_path.replace("\\", "/") if folder_path else ""
 
 
     # Sélectionner un fichier image unique
@@ -198,7 +155,7 @@ if os.name=='nt':
     import ctypes.wintypes
 
 
-    def select_file_ctypes(multi_select=False, file_filter="All Files (*.*)"):
+    def select_file_ctypes(multi_select=False, file_filter="All Files (*.*)", dialog_title="Select File(s)"):
         """
         Opens a native Windows file selection dialog with enforced modal-like behavior.
 
@@ -262,7 +219,7 @@ if os.name=='nt':
         ofn.lpstrFilter = filter_combined
         ofn.lpstrFile = ctypes.cast(file_buffer, ctypes.c_wchar_p)
         ofn.nMaxFile = MAX_PATH
-        ofn.lpstrTitle = "Select File(s)"
+        ofn.lpstrTitle = dialog_title
         ofn.Flags = OFN_EXPLORER | OFN_FILEMUSTEXIST
         if multi_select:
             ofn.Flags |= OFN_ALLOWMULTISELECT
@@ -291,12 +248,13 @@ if os.name=='nt':
         new_hwnds = list(set(hwnd_list_after) - set(hwnd_list_before))
 
         selector_hwnd = None
+        target_phrase = (dialog_title or "Select").lower()
         for hwnd in new_hwnds:
             length = user32.GetWindowTextLengthW(hwnd)
             if length > 0:
-                title = ctypes.create_unicode_buffer(length + 1)
-                user32.GetWindowTextW(hwnd, title, length + 1)
-                if "Select" in title.value:
+                window_title = ctypes.create_unicode_buffer(length + 1)
+                user32.GetWindowTextW(hwnd, window_title, length + 1)
+                if target_phrase in window_title.value.lower():
                     selector_hwnd = hwnd
                     break
 
@@ -335,165 +293,162 @@ if os.name=='nt':
 
 
 
-def select_new_file_ctypes(file_filter="All Files (*.*)"):
-    """
-    Opens a native Windows file dialog for creating a new file.
+    def select_new_file_ctypes(file_filter="All Files (*.*)", dialog_title="Create New File"):
+        """
+        Opens a native Windows file dialog for creating a new file.
 
-    Args:
-        file_filter (str): File type filters, e.g., "Text Files (*.txt)" or "All Files (*.*)".
+        Args:
+            file_filter (str): File type filters, e.g., "Text Files (*.txt)" or "All Files (*.*)".
 
-    Returns:
-        str: Selected file path as a string, or an empty string if cancelled.
-    """
-    OFN_EXPLORER = 0x00080000
-    MAX_PATH = 65536  # Large buffer for filename
+        Returns:
+            str: Selected file path as a string, or an empty string if cancelled.
+        """
+        OFN_EXPLORER = 0x00080000
+        MAX_PATH = 65536  # Large buffer for filename
 
-    class OPENFILENAME(ctypes.Structure):
-        _fields_ = [
-            ("lStructSize", ctypes.wintypes.DWORD),
-            ("hwndOwner", ctypes.wintypes.HWND),
-            ("hInstance", ctypes.wintypes.HINSTANCE),
-            ("lpstrFilter", ctypes.c_wchar_p),
-            ("lpstrCustomFilter", ctypes.c_wchar_p),
-            ("nMaxCustFilter", ctypes.wintypes.DWORD),
-            ("nFilterIndex", ctypes.wintypes.DWORD),
-            ("lpstrFile", ctypes.c_wchar_p),
-            ("nMaxFile", ctypes.wintypes.DWORD),
-            ("lpstrFileTitle", ctypes.c_wchar_p),
-            ("nMaxFileTitle", ctypes.wintypes.DWORD),
-            ("lpstrInitialDir", ctypes.c_wchar_p),
-            ("lpstrTitle", ctypes.c_wchar_p),
-            ("Flags", ctypes.wintypes.DWORD),
-            ("nFileOffset", ctypes.wintypes.WORD),
-            ("nFileExtension", ctypes.wintypes.WORD),
-            ("lpstrDefExt", ctypes.c_wchar_p),
-            ("lCustData", ctypes.wintypes.LPARAM),
-            ("lpfnHook", ctypes.c_void_p),
-            ("lpTemplateName", ctypes.c_wchar_p),
-            ("pvReserved", ctypes.c_void_p),
-            ("dwReserved", ctypes.wintypes.DWORD),
-            ("FlagsEx", ctypes.wintypes.DWORD),
-        ]
+        class OPENFILENAME(ctypes.Structure):
+            _fields_ = [
+                ("lStructSize", ctypes.wintypes.DWORD),
+                ("hwndOwner", ctypes.wintypes.HWND),
+                ("hInstance", ctypes.wintypes.HINSTANCE),
+                ("lpstrFilter", ctypes.c_wchar_p),
+                ("lpstrCustomFilter", ctypes.c_wchar_p),
+                ("nMaxCustFilter", ctypes.wintypes.DWORD),
+                ("nFilterIndex", ctypes.wintypes.DWORD),
+                ("lpstrFile", ctypes.c_wchar_p),
+                ("nMaxFile", ctypes.wintypes.DWORD),
+                ("lpstrFileTitle", ctypes.c_wchar_p),
+                ("nMaxFileTitle", ctypes.wintypes.DWORD),
+                ("lpstrInitialDir", ctypes.c_wchar_p),
+                ("lpstrTitle", ctypes.c_wchar_p),
+                ("Flags", ctypes.wintypes.DWORD),
+                ("nFileOffset", ctypes.wintypes.WORD),
+                ("nFileExtension", ctypes.wintypes.WORD),
+                ("lpstrDefExt", ctypes.c_wchar_p),
+                ("lCustData", ctypes.wintypes.LPARAM),
+                ("lpfnHook", ctypes.c_void_p),
+                ("lpTemplateName", ctypes.c_wchar_p),
+                ("pvReserved", ctypes.c_void_p),
+                ("dwReserved", ctypes.wintypes.DWORD),
+                ("FlagsEx", ctypes.wintypes.DWORD),
+            ]
 
-    user32 = ctypes.windll.user32
-    comdlg32 = ctypes.windll.comdlg32
+        user32 = ctypes.windll.user32
+        comdlg32 = ctypes.windll.comdlg32
 
-    # Convert file_filter into Windows format: "Description\0Pattern\0..."
-    if "(" in file_filter and ")" in file_filter:
-        description = file_filter.split("(")[0].strip()
-        pattern = file_filter[file_filter.find("(")+1:file_filter.find(")")]
-    else:
-        description = "All Files"
-        pattern = "*.*"
-
-    # Extract first extension for default
-    first_ext = pattern.split(";")[0].strip().lstrip("*.")
-    if not first_ext:
-        first_ext = "txt"
-
-    filter_combined = f"{description}\0{pattern}\0All Files\0*.*\0\0"
-
-    file_buffer = ctypes.create_unicode_buffer(MAX_PATH)
-
-    ofn = OPENFILENAME()
-    ofn.lStructSize = ctypes.sizeof(OPENFILENAME)
-    ofn.hwndOwner = 0
-    ofn.lpstrFilter = filter_combined
-    ofn.lpstrFile = ctypes.cast(file_buffer, ctypes.c_wchar_p)
-    ofn.nMaxFile = MAX_PATH
-    ofn.lpstrTitle = "Create New File"
-    ofn.Flags = OFN_EXPLORER
-    ofn.lpstrDefExt = first_ext  # <- extension ajoutée automatiquement si absente
-
-    # Mémoriser les fenêtres existantes
-    hwnd_list_before = []
-    def enum_windows_proc(hwnd, lParam):
-        hwnd_list_before.append(hwnd)
-        return True
-
-    EnumWindows = user32.EnumWindows
-    EnumWindowsProc = ctypes.WINFUNCTYPE(ctypes.wintypes.BOOL, ctypes.wintypes.HWND, ctypes.wintypes.LPARAM)
-    EnumWindows(EnumWindowsProc(enum_windows_proc), 0)
-
-    # Ouvre la boîte de dialogue
-    if not comdlg32.GetSaveFileNameW(ctypes.byref(ofn)):
-        return ""
-
-    time.sleep(0.1)
-
-    # Fenêtres après
-    hwnd_list_after = []
-    EnumWindows(EnumWindowsProc(lambda hwnd, lParam: hwnd_list_after.append(hwnd) or True), 0)
-    new_hwnds = list(set(hwnd_list_after) - set(hwnd_list_before))
-
-    selector_hwnd = None
-    for hwnd in new_hwnds:
-        length = user32.GetWindowTextLengthW(hwnd)
-        if length > 0:
-            title = ctypes.create_unicode_buffer(length + 1)
-            user32.GetWindowTextW(hwnd, title, length + 1)
-            if "Save" in title.value:
-                selector_hwnd = hwnd
-                break
-
-    def enforce_modal(hwnd):
-        if not hwnd:
-            return
-        while user32.IsWindow(hwnd):
-            user32.SetForegroundWindow(hwnd)
-            user32.SetWindowPos(hwnd, -1, 0, 0, 0, 0,
-                                0x0002 | 0x0001 | 0x0040)  # SWP_NOMOVE | SWP_NOSIZE | SWP_SHOWWINDOW
-            time.sleep(0.2)
-
-    if selector_hwnd:
-        threading.Thread(target=enforce_modal, args=(selector_hwnd,), daemon=True).start()
-
-    result = ctypes.wstring_at(ctypes.addressof(file_buffer), file_buffer._length_)
-    return result.split('\0', 1)[0].replace("\\", "/")
-
-
-
-import subprocess
-import os
-import sys
-
-def mac_shellcopy(src, dest):
-    """
-    Copy files or directories on macOS using AppleScript to show a native progress dialog.
-    :param src: Path to source file or directory (str or list of str)
-    :param dest: Path to destination directory (str)
-    :returns: True if successful, False otherwise
-    """
-    if isinstance(src, str):
-        src = [src]
-    src = [os.path.abspath(s) for s in src]
-    dest = os.path.abspath(dest)
-
-    # AppleScript for copying with progress dialog
-    script = f'''
-    set srcList to {{{", ".join([f'POSIX file "{s}"' for s in src])}}}
-    set destFolder to POSIX file "{dest}"
-    tell application "Finder"
-        repeat with aSrc in srcList
-            duplicate aSrc to destFolder with replacing
-        end repeat
-    end tell
-    '''
-
-    try:
-        result = subprocess.run(
-            ['osascript', '-e', script],
-            capture_output=True,
-            text=True
-        )
-        if result.returncode == 0:
-            return True
+        # Convert file_filter into Windows format: "Description\0Pattern\0..."
+        if "(" in file_filter and ")" in file_filter:
+            description = file_filter.split("(")[0].strip()
+            pattern = file_filter[file_filter.find("(")+1:file_filter.find(")")]
         else:
-            print("AppleScript error:", result.stderr)
-            return False
-    except Exception as e:
-        print("Error:", e)
-        return False
+            description = "All Files"
+            pattern = "*.*"
+
+        # Extract first extension for default
+        first_ext = pattern.split(";")[0].strip().lstrip("*.")
+        if not first_ext:
+            first_ext = "txt"
+
+        filter_combined = f"{description}\0{pattern}\0All Files\0*.*\0\0"
+
+        file_buffer = ctypes.create_unicode_buffer(MAX_PATH)
+
+        ofn = OPENFILENAME()
+        ofn.lStructSize = ctypes.sizeof(OPENFILENAME)
+        ofn.hwndOwner = 0
+        ofn.lpstrFilter = filter_combined
+        ofn.lpstrFile = ctypes.cast(file_buffer, ctypes.c_wchar_p)
+        ofn.nMaxFile = MAX_PATH
+        ofn.lpstrTitle = dialog_title
+        ofn.Flags = OFN_EXPLORER
+        ofn.lpstrDefExt = first_ext  # <- extension ajoutée automatiquement si absente
+
+        # Mémoriser les fenêtres existantes
+        hwnd_list_before = []
+        def enum_windows_proc(hwnd, lParam):
+            hwnd_list_before.append(hwnd)
+            return True
+
+        EnumWindows = user32.EnumWindows
+        EnumWindowsProc = ctypes.WINFUNCTYPE(ctypes.wintypes.BOOL, ctypes.wintypes.HWND, ctypes.wintypes.LPARAM)
+        EnumWindows(EnumWindowsProc(enum_windows_proc), 0)
+
+        # Ouvre la boîte de dialogue
+        if not comdlg32.GetSaveFileNameW(ctypes.byref(ofn)):
+            return ""
+
+        time.sleep(0.1)
+
+        # Fenêtres après
+        hwnd_list_after = []
+        EnumWindows(EnumWindowsProc(lambda hwnd, lParam: hwnd_list_after.append(hwnd) or True), 0)
+        new_hwnds = list(set(hwnd_list_after) - set(hwnd_list_before))
+
+        selector_hwnd = None
+        target_phrase = (dialog_title or "Save").lower()
+        for hwnd in new_hwnds:
+            length = user32.GetWindowTextLengthW(hwnd)
+            if length > 0:
+                window_title = ctypes.create_unicode_buffer(length + 1)
+                user32.GetWindowTextW(hwnd, window_title, length + 1)
+                if target_phrase in window_title.value.lower():
+                    selector_hwnd = hwnd
+                    break
+
+        def enforce_modal(hwnd):
+            if not hwnd:
+                return
+            while user32.IsWindow(hwnd):
+                user32.SetForegroundWindow(hwnd)
+                user32.SetWindowPos(hwnd, -1, 0, 0, 0, 0,
+                                    0x0002 | 0x0001 | 0x0040)  # SWP_NOMOVE | SWP_NOSIZE | SWP_SHOWWINDOW
+                time.sleep(0.2)
+
+        if selector_hwnd:
+            threading.Thread(target=enforce_modal, args=(selector_hwnd,), daemon=True).start()
+
+        result = ctypes.wstring_at(ctypes.addressof(file_buffer), file_buffer._length_)
+        return result.split('\0', 1)[0].replace("\\", "/")
+
+
+    def BoxMessage(text: str, mode: int = 0, title: str = None) -> int:
+        """
+        Affiche une boîte native Windows (via ctypes).
+        mode = 0 -> Error
+        mode = 1 -> Warning
+        mode = 2 -> Info
+        Retourne l'ID du bouton cliqué (IDOK = 1).
+        usage     BoxMessage("Ceci est une erreur", 0)
+        BoxMessage("Ceci est un avertissement", 1)
+        BoxMessage("Ceci est une information", 2)
+        BoxMessage("Ceci est une erreur", 0,"titr1")
+        BoxMessage("Ceci est un avertissement", 1,"titr2")
+        BoxMessage("Ceci est une information", 2,"titr3")
+
+
+        """
+
+
+        # Flags boutons
+        MB_OK = 0x00000000
+        MB_SYSTEMMODAL   = 0x00001000
+        MB_SETFOREGROUND = 0x00010000
+        MB_TOPMOST       = 0x00040000
+
+        # Icônes selon mode
+        if mode == 0:  # Error
+            icon_flag = 0x00000010  # MB_ICONERROR
+            if title is None: title = "Error"
+        elif mode == 1:  # Warning
+            icon_flag = 0x00000030  # MB_ICONWARNING
+            if title is None: title = "Warning"
+        elif mode == 2:  # Info
+            icon_flag = 0x00000040  # MB_ICONINFORMATION
+            if title is None: title = "Information"
+        else:
+            raise ValueError("mode doit être 0=Error, 1=Warning, 2=Info")
+
+        flags = MB_OK | icon_flag | MB_SYSTEMMODAL | MB_SETFOREGROUND | MB_TOPMOST
+
+        return windll.user32.MessageBoxW(0, c_wchar_p(text), c_wchar_p(title), flags)
 
-# Usage:
-# mac_shellcopy("/path/to/source", "/path/to/destination")
