@@ -498,4 +498,8 @@ if __name__ == '__main__':
     app = QApplication(sys.argv)
     window = OWAAITResourcesManager()
     window.show()
-    app.exec_()
+    if hasattr(app, "exec"):
+        app.exec()
+    else:
+        app.exec_()
+
