@@ -22,6 +22,7 @@ class OWRuleViewer(widget.OWWidget):
     name = "CN2 Rule Selector"
     description = "Review rules induced from data."
     icon = "icons/CN2RuleViewer.svg"
+    category = "AAIT - ALGORITHM"
     if "site-packages/Orange/widgets" in os.path.dirname(os.path.abspath(__file__)).replace("\\", "/"):
         icon = "icons_dev/CN2RuleViewer.svg"
     priority = 1140
@@ -205,8 +206,8 @@ class OWRuleViewer(widget.OWWidget):
 #                self.Outputs.annotated_data.send(outdata_status)
 
             # a verifier surement fausse alarme pyflakes
-            data_output = data.from_table_rows(data, selected_indices) \
-                if len(selected_indices) else None
+            # data_output = data.from_table_rows(data, selected_indices) \
+            #     if len(selected_indices) else None
         if self.classifier!=None:
             if self.selected!=[] and self.selected is not None:
                 data = []
