@@ -3,7 +3,7 @@ import re
 from typing import Any, Iterable, List, Set
 from pygments.formatters import HtmlFormatter
 import markdown
-
+import sys
 # Dans la classe OWCorpusViewer, ajoutez les méthodes suivantes :
 import numpy as np
 from AnyQt.QtCore import (
@@ -281,6 +281,7 @@ class OWCorpusViewer(OWWidget, ConcurrentWidgetMixin):
     name = "MarkDown Viewer"
     description = "Display corpus contents with Markdown."
     icon = "icons/MDViewer.png"
+    category = "AAIT - TOOLBOX"
     if "site-packages/Orange/widgets" in os.path.dirname(os.path.abspath(__file__)).replace("\\", "/"):
         icon = "icons_dev/MDViewer.png"
     priority = 1002
@@ -735,8 +736,6 @@ class OWCorpusViewer(OWWidget, ConcurrentWidgetMixin):
 
         def replace_path(match):
             image_path = match.group(1)
-            original_path = image_path
-
 
             # Si l'image est locale, corriger le chemin
             if not image_path.startswith(('http://', 'https://')):
@@ -830,15 +829,10 @@ class OWCorpusViewer(OWWidget, ConcurrentWidgetMixin):
             )
 
 if __name__ == "__main__":
-    pass
-    # from orangewidget.utils.widgetpreview import WidgetPreview
-
-    # from orangecontrib.text.preprocess import BASE_TOKENIZER
-
-    # corpus_ = Corpus.from_file(r"C:\Users\timot\Desktop\exemple sortie md.tab")
-    # #corpus_ = Corpus.from_file("book-excerpts")
-    # #corpus_ = corpus_[:3]
-    # corpus_ = BASE_TOKENIZER(corpus_)
-    # WidgetPreview(OWCorpusViewer).run(corpus_)
-
-    # TODO: Implement a test that works on all platform.
\ No newline at end of file
+    app = QApplication(sys.argv)
+    obj = OWCorpusViewer()
+    obj.show()
+    if hasattr(app, "exec"):
+        app.exec()
+    else:
+        app.exec_()
\ No newline at end of file
