@@ -1,6 +1,6 @@
 import os
 from AnyQt.QtWidgets import QPushButton, QApplication, QRadioButton, QComboBox,QCheckBox,QLineEdit
-
+import sys
 
 from Orange.widgets.settings import Setting
 from Orange.widgets.widget import OWWidget, Output, Input
@@ -8,10 +8,10 @@ from Orange.data import Table, Domain, StringVariable
 
 
 if "site-packages/Orange/widgets" in os.path.dirname(os.path.abspath(__file__)).replace("\\", "/"):
-    from Orange.widgets.orangecontrib.AAIT.utils import windows_utils,MetManagement
+    from Orange.widgets.orangecontrib.AAIT.utils import windows_utils,MetManagement,mac_utils
     from Orange.widgets.orangecontrib.AAIT.utils.import_uic import uic
 else:
-    from orangecontrib.AAIT.utils import windows_utils, MetManagement
+    from orangecontrib.AAIT.utils import windows_utils, MetManagement,mac_utils
     from orangecontrib.AAIT.utils.import_uic import uic
 
 
@@ -21,6 +21,7 @@ class OWInputSelector(OWWidget):
     icon = "icons/in_or_out.png"
     if "site-packages/Orange/widgets" in os.path.dirname(os.path.abspath(__file__)).replace("\\", "/"):
         icon = "icons_dev/in_or_out.png"
+    category = "AAIT - TOOLBOX"
     gui = os.path.join(os.path.dirname(os.path.abspath(__file__)), "designer/ow_in_or_out_path.ui")
     priority = 10
     want_control_area = False
@@ -29,6 +30,7 @@ class OWInputSelector(OWWidget):
 
     class Outputs:
         path = Output("out data", Table)
+        path_string = Output("Data (str)", str, auto_summary=False)
 
 
     select_folder_radio_button_checked: str = Setting("True")
@@ -36,6 +38,12 @@ class OWInputSelector(OWWidget):
     new_file:str = Setting("False")
     combo_box_current_text: str=Setting("All (*.*)")
     line_edit_value:str=Setting("")
+    dialog_window_title: str = Setting("")
+    str_checkBox_3_send: str=Setting("True")
+    str_Launch_automatically_on_startup: str=Setting("False")
+
+
+
     def update_qt_view_from_settings(self):
         if str(self.select_folder_radio_button_checked) =="True":
             self.radioButton.setChecked(True)
@@ -43,6 +51,7 @@ class OWInputSelector(OWWidget):
             self.checkBox.hide()
             self.comboBox.hide()
             self.checkBox_new_file.hide()
+            self.update_dialog_title_placeholder()
             return
         self.radioButton.setChecked(False)
         self.radioButton2.setChecked(True)
@@ -59,21 +68,44 @@ class OWInputSelector(OWWidget):
             self.checkBox_new_file.setChecked(True)
             self.checkBox.setEnabled(False)
 
-
-
-
-
         index = self.comboBox.findText(str(self.combo_box_current_text))
         if index != -1:
             self.comboBox.setCurrentIndex(index)
         else:
             self.comboBox.setCurrentIndex(0)  # Sélectionner le premier item
 
+
+        if self.str_checkBox_3_send != "False":
+            self.checkBox_3_send.setChecked(True)
+        else:
+            self.checkBox_3_send.setChecked(False)
+
+        if self.str_Launch_automatically_on_startup != "False":
+            self.checkBox_launch_at_startup.setChecked(True)
+        else:
+            self.checkBox_launch_at_startup.setChecked(False)
+
+
+
+
+
         self.checkBox.show()
         self.comboBox.show()
         self.checkBox_new_file.show()
+        self.update_dialog_title_placeholder()
+
 
     def update_setting_from_qt_view(self):
+        if self.checkBox_launch_at_startup.isChecked():
+            self.str_Launch_automatically_on_startup = "True"
+        else:
+            self.str_Launch_automatically_on_startup ="False"
+
+
+        if self.checkBox_3_send.isChecked():
+            self.str_checkBox_3_send = "True"
+        else:
+            self.str_checkBox_3_send ="False"
         if self.radioButton.isChecked():
             self.select_folder_radio_button_checked=True
             self.update_qt_view_from_settings()
@@ -108,7 +140,7 @@ class OWInputSelector(OWWidget):
 
 
         self.setFixedWidth(700)
-        self.setFixedHeight(300)
+        self.setFixedHeight(400)
         uic.loadUi(self.gui, self)
         self.radioButton=self.findChild(QRadioButton, 'radioButton')
         self.radioButton2 = self.findChild(QRadioButton, 'radioButton_2')
@@ -118,6 +150,28 @@ class OWInputSelector(OWWidget):
         self.pushButton = self.findChild(QPushButton, 'pushButton')
         self.lineEdit=self.findChild(QLineEdit,'lineEdit')
         self.lineEdit.setText(self.line_edit_value)
+        self.windowTitleEdit = self.findChild(QLineEdit, 'lineEdit_window_title')
+        self.checkBox_3_send = self.findChild(QCheckBox, 'checkBox_3_send')
+        self.checkBox_launch_at_startup = self.findChild(QCheckBox, 'checkBox_launch_at_startup')
+
+        if self.str_checkBox_3_send=="True":
+            self.checkBox_3_send.setChecked(True)
+        else:
+            self.checkBox_3_send.setChecked(False)
+
+
+        if self.str_Launch_automatically_on_startup=="True":
+            self.checkBox_launch_at_startup.setChecked(True)
+        else:
+            self.checkBox_launch_at_startup.setChecked(False)
+
+        if self.windowTitleEdit is not None:
+            sanitized_title = (self.dialog_window_title or "").strip()
+            if sanitized_title != self.dialog_window_title:
+                self.dialog_window_title = sanitized_title
+            self.windowTitleEdit.setText(sanitized_title)
+            self.update_dialog_title_placeholder()
+            self.windowTitleEdit.editingFinished.connect(self.update_dialog_title_setting)
 
         # based on the string you can modify orders and add without problem
         file_types = [
@@ -216,16 +270,19 @@ class OWInputSelector(OWWidget):
             "Fichier WOFF (*.woff;*.WOFF)",
             "Fichier WOFF2 (*.woff2;*.WOFF2)",
             "Fichier EOT (*.eot;*.EOT)",
-            "Fichier Markdown (*.md;*.markdown;*.MD;*.MARKDOWN)"
+            "Fichier Markdown (*.md;*.markdown;*.MD;*.MARKDOWN)",
+            "Fichier GGUF (*.gguf;*.GGUF)"
         ]
         self.comboBox.addItems(file_types)
 
-
         self.update_qt_view_from_settings()
         self.radioButton.clicked.connect(self.update_setting_from_qt_view)
         self.radioButton2.clicked.connect(self.update_setting_from_qt_view)
         self.checkBox.clicked.connect(self.update_setting_from_qt_view)
         self.checkBox_new_file.clicked.connect(self.update_setting_from_qt_view)
+        self.checkBox_3_send.clicked.connect(self.update_setting_from_qt_view)
+        self.checkBox_launch_at_startup.clicked.connect(self.update_setting_from_qt_view)
+
 
         self.comboBox.currentTextChanged.connect(self.on_text_changed)
         self.pushButton.clicked.connect(self.launch_fucntionalities)
@@ -246,9 +303,15 @@ class OWInputSelector(OWWidget):
             self.checkBox.setEnabled(False)
             self.checkBox_new_file.setEnabled(False)
             self.comboBox.setEnabled(False)
+        if self.str_Launch_automatically_on_startup=="True":
+            self.launch_fucntionalities()
 
     def update_line_edit_setting(self):
-        self.line_edit_value=MetManagement.TransfromPathToStorePath(str(self.lineEdit.text()))
+        def strip_quotes(s: str) -> str:
+            if len(s) >= 1 and s[0] == s[-1] and s[0] in ("'", '"'):
+                return s[1:-1]
+            return s
+        self.line_edit_value=MetManagement.TransfromPathToStorePath(strip_quotes(str(self.lineEdit.text())))
         if (len(self.line_edit_value)) == 0:
             self.radioButton.setEnabled(True)
             self.radioButton2.setEnabled(True)
@@ -265,25 +328,53 @@ class OWInputSelector(OWWidget):
             self.checkBox_new_file.setEnabled(False)
             self.comboBox.setEnabled(False)
 
+    def update_dialog_title_setting(self):
+        if getattr(self, 'windowTitleEdit', None) is None:
+            return
+        value = self.windowTitleEdit.text().strip()
+        if self.windowTitleEdit.text() != value:
+            self.windowTitleEdit.setText(value)
+        self.dialog_window_title = value
+        self.update_dialog_title_placeholder()
+
+    def get_default_dialog_title(self):
+        if str(self.select_folder_radio_button_checked) == "True":
+            return "Select a directory"
+        if str(self.new_file) == "True":
+            return "Create New File"
+        return "Select File(s)"
+
+    def get_effective_dialog_title(self):
+        default_title = self.get_default_dialog_title()
+        if self.dialog_window_title:
+            value = self.dialog_window_title.strip()
+            if value:
+                return value
+        return default_title
+
+    def update_dialog_title_placeholder(self):
+        if getattr(self, 'windowTitleEdit', None) is None:
+            return
+        self.windowTitleEdit.setPlaceholderText(self.get_default_dialog_title())
+
 
     @Inputs.input_table
     def input_table(self, data):
         self.in_data = data
+        # if self.str_checkBox_3_send=="False":
+        #     return
         if data is not None:
             self.launch_fucntionalities()
 
 
 
-
-
-
     def launch_fucntionalities(self):
         if self.line_edit_value!="":
             self.selected_path=MetManagement.TransfromStorePathToPath(self.line_edit_value)
             self.commit_path()
             return
         if str(self.select_folder_radio_button_checked) =="True":
-            self.selected_path=self.select_folder()
+            self.selected_path = self.select_folder(self.get_effective_dialog_title())
             if self.selected_path is None:
                 self.selected_path=""
             if self.selected_path=="":
@@ -294,10 +385,11 @@ class OWInputSelector(OWWidget):
         if str(self.select_multiple_file_checked)!="False":
             multi_select=True
         file_filter=str(self.combo_box_current_text)
+        dialog_title = self.get_effective_dialog_title()
         if str(self.new_file)=="False":
-            self.selected_path=self.select_file(multi_select,file_filter)
+            self.selected_path = self.select_file(multi_select, file_filter, dialog_title)
         else:
-            self.selected_path = self.select_new_file(file_filter)
+            self.selected_path = self.select_new_file(file_filter, dialog_title)
         if self.selected_path is None:
             self.selected_path = ""
         if self.selected_path == "":
@@ -305,42 +397,54 @@ class OWInputSelector(OWWidget):
         self.commit_path()
         return
 
-    def select_file(self,multi_select,file_filtter):
+    def select_file(self, multi_select, file_filter, dialog_title):
         if os.name == 'nt':
-            return windows_utils.select_file_ctypes(multi_select=multi_select, file_filter=file_filtter)
-        print("a faire correctement sans qt sur mac (et linux")
+            return windows_utils.select_file_ctypes(
+                multi_select=multi_select,
+                file_filter=file_filter,
+                dialog_title=dialog_title
+            )
+        if sys.platform.startswith("Darwin"):
+            return  mac_utils.select_file_macos(
+                multi_select=multi_select,
+                file_filter=file_filter,
+                dialog_title=dialog_title
+            )
         from AnyQt.QtWidgets import QFileDialog
         files, _ = QFileDialog.getOpenFileNames(
             self,  # parent
-            "Sélectionner des fichiers",  # titre de la fenêtre
-            "",  # répertoire initial
-            "Tous les fichiers (*.*)"  # filtre
+            dialog_title,  # window title
+            "",  # initial directory
+            file_filter  # filter
         )
         return files
 
-    def select_new_file(self,file_filtter):
+    def select_new_file(self, file_filter, dialog_title):
         if os.name == 'nt':
-            return windows_utils.select_new_file_ctypes(file_filter=file_filtter)
+            return windows_utils.select_new_file_ctypes(file_filter=file_filter, dialog_title=dialog_title)
+        if sys.platform.startswith("Darwin"):
+            return mac_utils.select_new_file_macos(file_filter=file_filter, dialog_title=dialog_title)
+
+
         print("a faire correctement sans qt sur mac (et linux")
         from AnyQt.QtWidgets import QFileDialog
         files, _ = QFileDialog.getSaveFileName(
             self,  # parent
-            "Créer un fichier",  # titre de la fenêtre
-            "",  # répertoire initial
-            "Tous les fichiers (*.*)"  # filtre
+            dialog_title,  # window title
+            "",  # initial directory
+            file_filter  # filter
         )
         return str(files)
 
-
-
-
-    def select_folder(self):
+    def select_folder(self, dialog_title):
         if os.name=='nt':
-            return windows_utils.select_folder_ctypes()
+            return windows_utils.select_folder_ctypes(title=dialog_title)
+        if sys.platform.startswith("Darwin"):
+            return mac_utils.select_folder_macos(title=dialog_title)
+
         # a faire correctement sans qt sur mac
-        print("a faire correctement sans qt sur mac (et linux")
         from AnyQt.QtWidgets import QFileDialog
-        return QFileDialog.getExistingDirectory(self, "Sélectionner un dossier")
+        return QFileDialog.getExistingDirectory(self, dialog_title)
 
     def commit_path(self):
         self.error("")
@@ -348,6 +452,7 @@ class OWInputSelector(OWWidget):
             if "path" in self.in_data.domain or "path" in self.in_data.domain.metas:
                 self.error("Path déjà dans les domaines d entrées.")
                 self.Outputs.path.send(None)
+                self.Outputs.path_string.send(None)
 
         if not self.selected_path:
             return
@@ -369,11 +474,12 @@ class OWInputSelector(OWWidget):
             new_table.metas[0] = [self.selected_path]
 
         self.Outputs.path.send(new_table)
+        self.Outputs.path_string.send(self.selected_path)
 
 
 # Test standalone
 if __name__ == "__main__":
-    import sys
+
     app = QApplication(sys.argv)
     window = OWInputSelector()
     window.show()
